/* -*- mode: c++; c-basic-offset: 3 -*- */
#ifndef RDS_RESAMPLE_FRAME_HH
#define RDS_RESAMPLE_FRAME_HH

#include "framecpp/Common/MD5Sum.hh"
#include "framecpp/FrameH.hh"
#include "framecpp/FrVect.hh"

#include "filereader.hh"
#include "rdsframe.hh"   


namespace Filters   
{
   class ResampleBase;
}

   
//-----------------------------------------------------------------------------
//   
//: Functional to create RDS resampled frames.
//   
class ResampleRawFrame : private RDSFrame
{
public:
   
   ResampleRawFrame( const char* frame_files, 
                     const char* channels,
                     const char* resample,
		     const RDSFrame::Options& UserOptions );

   virtual ~ResampleRawFrame();
   
   // Expose the main hook for processing information
   using RDSFrame::ProcessRequest;

protected:
   virtual void processChannel( INT_4U ChannelOffset,
				fr_adc_data_type Adc );

   virtual void processChannel( INT_4U ChannelOffset,
				fr_proc_data_type Proc );
			
private:
   // Useful typedefs:
   typedef std::vector< Filters::ResampleBase* >::const_iterator
   state_const_iterator;
   typedef std::vector< Filters::ResampleBase* >::iterator 
   state_iterator;   

#if 0
   std::string	m_next_filename;
   std::string	m_current_filename;
#endif /* 0 */

   //: String representation of all resample factors.
   std::string mResampleRecord;
   
   //: Vector of all resample factors.
   std::vector< INT_4U > mResampleFactor;      

   //: Vector of base objects used for channel resampling.
   std::vector< Filters::ResampleBase* > mResampleBase;

   //: Index of the current channel
   INT_4U	mChannelOffset;

   //: Dt of the current frame
   REAL_8 mCurrentDt;
   
   //: Current new frame
   frame_h_type			mCurrentFrame;
   
#if 0
   std::auto_ptr< FileWriter >	mCurrentFileWriter;
#endif /* 0 */

   //: Boolean for state of seeding resampling stuctures.
   bool		mSeedState;

   bool		m_should_write;

   INT_4U	m_is_last_frame_of_file;

#if 0
   General::GPSTime	m_frame_file_start_time;
#endif /* 0 */
   REAL_8		m_frame_file_deltaT;

   // Create frame
   virtual void createHistory( );
   
   // Initialization
   void initResampleFactor( );

   // Mutators
   void adjustChannelData();   
   
   void adjustResultChannel( const std::string& name,
			     const Filters::ResampleBase* state );

#if 0
   // Open a frame file
   virtual void openFrameFile( const std::string& Filename );
#endif /* 0 */

   virtual void writeFrameToFile( );
};

   
#endif   
